Attribute VB_Name = "CircleStuff"
Option Explicit

Public Const NAN As Single = 1E+20

' Find the points where the two circles intersect.
Public Function FindCircleCircleIntersections( _
    ByVal center0 As PointF, ByVal radius0 As Single, _
    ByVal center1 As PointF, ByVal radius1 As Single, _
    ByRef intersection1 As PointF, _
    ByRef intersection2 As PointF) As Integer
Dim dx As Single
Dim dy As Single
Dim dist As Double
Dim a As Double
Dim h As Double
Dim cx2 As Double
Dim cy2 As Double

    ' Find the distance between the centers.
    dx = center0.X - center1.X
    dy = center0.Y - center1.Y
    dist = Sqr(dx * dx + dy * dy)

    ' See how many solutions there are.
    If (dist > radius0 + radius1) Then
        ' No solutions, the circles are too far apart.
        intersection1.X = NAN
        intersection1.Y = NAN
        intersection2.X = NAN
        intersection2.Y = NAN
        FindCircleCircleIntersections = 0
        Exit Function
    ElseIf (dist < Math.Abs(radius0 - radius1)) Then
        ' No solutions, one circle contains the other.
        intersection1.X = NAN
        intersection1.Y = NAN
        intersection2.X = NAN
        intersection2.Y = NAN
        FindCircleCircleIntersections = 0
    ElseIf ((dist = 0) And (radius0 = radius1)) Then
        ' No solutions, the circles coincide.
        intersection1.X = NAN
        intersection1.Y = NAN
        intersection2.X = NAN
        intersection2.Y = NAN
        FindCircleCircleIntersections = 0
    Else
        ' Find a and h.
        a = (radius0 * radius0 - _
            radius1 * radius1 + dist * dist) / (2 * dist)
        h = Sqr(radius0 * radius0 - a * a)

        ' Find P2.
        cx2 = center0.X + a * (center1.X - center0.X) / dist
        cy2 = center0.Y + a * (center1.Y - center0.Y) / dist

        ' Get the points P3.
        Set intersection1 = New PointF
        Set intersection2 = New PointF
        intersection1.X = CSng(cx2 + h * (center1.Y - center0.Y) / dist)
        intersection1.Y = CSng(cy2 - h * (center1.X - center0.X) / dist)
        intersection2.X = CSng(cx2 - h * (center1.Y - center0.Y) / dist)
        intersection2.Y = CSng(cy2 + h * (center1.X - center0.X) / dist)

        ' See if we have 1 or 2 solutions.
        If (dist = radius0 + radius1) Then
            FindCircleCircleIntersections = 1
        Else
            FindCircleCircleIntersections = 2
        End If
        Exit Function
    End If
End Function

' Find the tangent points for this circle and external point.
' Return true if we find the tangents, false if the point is
' inside the circle.
Public Function FindTangents( _
    ByVal center As PointF, ByVal radius As Single, _
    ByVal pt As PointF, _
    ByRef pt1 As PointF, ByRef pt2 As PointF) As Boolean
Dim dx As Double
Dim dy As Double
Dim D_squared As Double
Dim L As Double

    ' Find the distance squared from the
    ' external point to the circle's center.
    dx = center.X - pt.X
    dy = center.Y - pt.Y
    D_squared = dx * dx + dy * dy
    If (D_squared < radius * radius) Then
        pt1.X = -1
        FindTangents = False
        Exit Function
    End If

    ' Find the distance from the external point
    ' to the tangent points.
    L = Sqr(D_squared - radius * radius)

    ' Find the points of intersection between
    ' the original circle and the circle with
    ' center external_point and radius dist.
    FindCircleCircleIntersections _
        center, radius, pt, CSng(L), pt1, pt2

    FindTangents = True
End Function


